<?php

/**
 * @file
 *
 */

/**
 * Class feeds_callback_fetcher
 */
class feeds_callback_fetcher extends FeedsFileFetcher {

  /**
   * Fetch content from a source and return it.
   *
   * @param \FeedsSource $source
   *
   * @return \FeedsFileFetcherResult
   * @throws \Exception
   */
  public function fetch(FeedsSource $source) {
    $source_config = $source->getConfigFor($this);

    if (!function_exists($source_config['callback'])) {
      throw new Exception(t('Callback @callback doesn\'t exists.', array('@callback' => $source_config['callback'])));
    }

    $file = drupal_tempnam(file_directory_temp(), 'feeds-');
    file_put_contents($file, call_user_func($source_config['callback']));

    if (0 == filesize($file)) {
      throw new Exception(t('Callback returned nothing, temporary file size is empty.'));
    }

    return new FeedsFileFetcherResult($file);
  }

  /**
   * Override parent::sourceDefaults().
   */
  public function sourceDefaults() {
    return array(
      'callback' => '',
      'source' => NULL,
    );
  }

  /**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    return array(
      'callback' => '',
      'source' => NULL,
    );
  }

  /**
   * Override parent::sourceForm().
   */
  public function sourceForm($form_state) {
    $form = array();

    $form['callback'] = array(
      '#type' => 'textfield',
      '#title' => t('Callback'),
      '#description' => t('Specify a callback'),
      '#default_value' => $this->config['callback'],
      '#required' => TRUE,
    );

    $form['source'] = array(
      '#type' => 'value',
      '#value' => NULL,
    );

    return $form;
  }

  /**
   * Override parent::configFormValidate().
   */
  public function sourceFormValidate(&$values) {
    $this->checkCallbackValidation($values);
  }

  /**
   * Override parent::configForm().
   */
  public function configForm(&$form_state) {
    $form = array();

    $form['callback'] = array(
      '#type' => 'textfield',
      '#title' => t('Callback'),
      '#description' => t('Specify a callback'),
      '#default_value' => $this->config['callback'],
      '#required' => TRUE,
    );

    $form['source'] = array(
      '#type' => 'value',
      '#value' => NULL,
    );

    return $form;
  }

  /**
   * Override parent::configFormValidate().
   */
  public function configFormValidate(&$values) {
    $this->checkCallbackValidation($values);
  }

  /**
   * Custom callback to validate the configuration.
   *
   * @param $values
   */
  private function checkCallbackValidation(&$values) {
    $values['callback'] = trim($values['callback']);
    if (!function_exists($values['callback'])) {
      form_set_error('feeds][feeds_callback_fetcher][callback', t('Unable to save, the callback does not exist.'));
    }
  }

}
